// Copyright (C) 2004 MySQL AB
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License version 2 as published by
// the Free Software Foundation
//
// There are special exceptions to the terms and conditions of the GPL 
// as it is applied to this software. View the full text of the 
// exception in file EXCEPTIONS in the directory of this software 
// distribution.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 

using System;
using System.Data;

namespace MySql.Data.MySqlClient
{
	/// <summary>
	/// Represents a SQL transaction to be made in a MySQL database. This class cannot be inherited.
	/// </summary>
	/// <include file='docs/MySqlTransaction.xml' path='MyDocs/MyMembers[@name="Class"]/*'/>
	public sealed class MySqlTransaction : IDbTransaction
	{
		private IsolationLevel	level;
		private MySqlConnection	conn;
		private bool			open;

		internal MySqlTransaction() 
		{
			open = true;
		}

		#region Properties

		/// <summary>
		/// Gets the <see cref="MySqlConnection"/> object associated with the transaction, or a null reference (Nothing in Visual Basic) if the transaction is no longer valid.
		/// </summary>
		public IDbConnection Connection
		{
			get { return conn;	} 
			set { conn = (MySqlConnection)value; }
		}

		/// <summary>
		/// Specifies the <see cref="IsolationLevel"/> for this transaction.
		/// </summary>
		public IsolationLevel IsolationLevel 
		{
			get { return level; }
			set { level = value; }
		}

		#endregion

		void System.IDisposable.Dispose() 
		{
		}

		/// <summary>
		/// Commits the database transaction.
		/// </summary>
		public void Commit()
		{
			if (conn == null || conn.State != ConnectionState.Open)
				throw new InvalidOperationException("Connection must be valid and open to commit transaction");
			if (!open)
				throw new InvalidOperationException("Transaction has already been committed or is not pending");
			try 
			{
				MySqlCommand cmd = new MySqlCommand( "COMMIT", conn );
				cmd.ExecuteNonQuery();
				open = false;
			}
			catch (MySqlException ex) 
			{
				throw ex;
			}
		}

		/// <summary>
		/// Overloaded. Rolls back a transaction from a pending state.
		/// </summary>
		public void Rollback()
		{
			if (conn == null || conn.State != ConnectionState.Open)
				throw new InvalidOperationException("Connection must be valid and open to commit transaction");
			if (!open)
				throw new InvalidOperationException("Transaction has already been rolled back or is not pending");
			try 
			{
				MySqlCommand cmd = new MySqlCommand( "ROLLBACK", conn );
				cmd.ExecuteNonQuery();
				open = false;
			}
			catch (MySqlException ex) 
			{
				throw ex;
			}
		}
	}
}
